//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/wavtool/datacopy.c,v $
// $Id: datacopy.c,v 1.7 2006/04/06 21:16:28 ah Exp $ 
//
// Description: cut a pice out of a wav file into a new wav file
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: datacopy.c,v 1.7 2006/04/06 21:16:28 ah Exp $ (c) 2000 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
#include <AH/IO/Convert.h>
USING_NAMESPACE(AH)

#include "datacopy.h"

#include <cstdlib>
#include <cmath>
#include <iostream>
#include <string>
USING_NAMESPACE(std)

// --- local constants ---

static const uint32 MAX_BUF_SIZE = 1 * 1024 * 1024;  // 1MB

// --------------- class DataCopyRaw ------------------------------------

//-----------------------------------------------------------------------
// copy specified data from input file to output file (based on samples)
// On Error: throws an exception
//-----------------------------------------------------------------------
void DataCopyRaw::copy
(
    int s1,   // start sample
    int s2    // end sample
)
{
    // get desired range (samples or times):

    uint32 samNum = in.getSampleNum();
    if (verbose > 5)
	cerr << "DataCopyRaw::copy(" << s1 << ", " << s2 
	     << ") from " << samNum << " samples" << endl;

    if (s1 < 0) s1 = 0;                    // -1 means begin of file
    in.setDataPos((uint32)s1);             // set read pointer
    samNum -= s1;
    
    uint32 endPos;                         // end position in file
    if (0 <= s2)
    {
	++s2; // prevent ladder problem
	endPos = in.getDataPos((uint32)s2);
	samNum -= in.getSampleNum() - s2;
    }
    else                                   // -1 means end of file
	endPos = in.getDataPos(in.getSampleNum());

    if (verbose > 4)
	cerr << "Taking " << samNum << " samples" << endl;

    // read desired data range:
    uint32 sizeToRead = endPos - in.getDataPos();

    // make a buffer for R/W of data (on stack):
    const uint32 bufSize = MAX_BUF_SIZE;
    unsigned char buf[bufSize];

    uint32 pos = 0;
    uint32 max = sizeToRead;
    do
    {
	// how much can we read (not more than buffer size) ?
	uint32 curBufSize = sizeToRead;
	if (curBufSize > bufSize) curBufSize = bufSize;

	// read input file data :
	int32 rdsize = readBuf(buf, curBufSize);
	if (0 >= rdsize) break;
	sizeToRead -= rdsize;

	// process data:
	processBuf(pos, max, buf, rdsize);

	// write output file data :
	if (0 >= writeBuf(buf, rdsize)) break;

	pos += rdsize;
    }
    while (sizeToRead);
} // end of copy_data()



// read data buffer
int DataCopyRaw::readBuf(unsigned char* buf, unsigned int size)
{
    if (4 < verbose) cerr << "read " << size 
			  << " bytes from input file \"" 
			  << inputFileName << "\":" << endl;
    int32 rdsize = in.readRaw(buf, size);
    return rdsize;
}

// write data buffer
int DataCopyRaw::writeBuf(const unsigned char* buf, unsigned int size)
{
    uint32 sizeWritten = 0;
    uint32 sizeToWrite = size;
    while (0 < sizeToWrite)
    {
	if (4 < verbose) cerr << "write " << sizeToWrite 
			      << " bytes into output file \"" 
			      << outputFileName << "\":" << endl;
	sizeWritten = out.writeRaw(buf, sizeToWrite);
	if (0 >= sizeWritten) break;
	sizeToWrite -= sizeWritten;
    }
    if (0 == sizeToWrite) return size;
    return sizeWritten;
}


// --------------- class DataCopyFade ----------------------------------

// process data buffer
void DataCopyFade::processBuf
(
    uint32 pos,
    uint32 max,
    unsigned char* buf,
    unsigned int size
) 
{
    // fix for 16 bit 2 channel stereo:
    uint32 posnum = pos/4;
    uint32 maxnum = max/4;
    unsigned int sizenum = size/2;

    if (verbose > 3) cerr << "Fade: " << posnum << "/" << maxnum << endl;
    if (verbose > 3) cerr << "Fade: sizenum=" << sizenum << endl;

    double rate = MAX_DB/(double)maxnum;
    uint32 sw_pos = maxnum - (uint32)(SW_DB / rate);
    double m = pow(10.0, -SW_DB * 0.05) / (double)sw_pos;

    for (unsigned int i = 0; i < sizenum; i += 2, buf += 4, ++posnum)
    {
	double factor = getFadeFactor(posnum, maxnum, rate, sw_pos, m);

        // WAV file contains little endian data
        // channel 1:
        int16 ampl = IOConvert::LittleEndCharsToInt16(buf);
        ampl = (int16)(((double)ampl * factor) + 0.5);
        IOConvert::Uint16ToLittleEndChars(ampl, buf);
        // channel 2:
        ampl = IOConvert::LittleEndCharsToInt16(buf + 2);
        ampl = (int16)(((double)ampl * factor) + 0.5);
        IOConvert::Uint16ToLittleEndChars(ampl, buf + 2);
    }
}


// --------------- class DataCopyFadeIn --------------------------------

double DataCopyFadeIn::getFadeFactor
(
    uint32 pos, 
    uint32 max, 
    double rate, 
    uint32 sw_pos, 
    double m
)
{
    // now check if we should switch to the linear part:
    bool calclog = true;
    if (pos < sw_pos) calclog = false;

    double factor;
    if (calclog)
    {
	double db = (double)(max - pos) * rate;
	factor = pow(10.0, -db * 0.05);
    }
    else
    {
	factor = m * (double)pos;
    }
    if ((verbose > 4) && (0 == (pos % 10000)))
	cerr << "Fade in at " << pos << "/" << max << " = " 
	     << factor*100 << "% " << (calclog ? "log":"lin") << endl;

    // check limits:
    if (factor < 0) factor = 0;
    else if (factor > 1) factor = 1;

    return factor;
}

// --------------- class DataCopyFadeOut -------------------------------

double DataCopyFadeOut::getFadeFactor
(
    uint32 pos, 
    uint32 max, 
    double rate, 
    uint32 sw_pos, 
    double m
)
{
    // now check if we should switch to the linear part:
    bool calclog = true;
    if (pos > (max - sw_pos)) calclog = false;

    double factor;
    if (calclog)
    {
	double db = (double)pos * rate;
	factor = pow(10.0, -db * 0.05);
    }
    else
    {
	factor = m * (double)(max - pos);
    }
    if ((verbose > 4) && (0 == (pos % 10000)))
	cerr << "Fade out at " << pos << "/" << max << " = " 
	     << factor*100 << "% " << (calclog ? "log":"lin") << endl;

    // check limits:
    if (factor < 0) factor = 0;
    else if (factor > 1) factor = 1;

    return factor;
}


// --- end of file ---









