//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/wavtool/wav2text.c,v $
// $Id: wav2text.c,v 1.9 2004/04/21 21:42:39 ah Exp $ 
//
// Description: cut a pice out of a wav file into a new wav file
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: wav2text.c,v 1.9 2004/04/21 21:42:39 ah Exp $ (c) 2000 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)

#include <cstdlib>
#ifdef NO_UNISTD
#   include <getopt.h>
#else
#   include <unistd.h>
#endif
#include <string>
#include <cerrno>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <iostream>
#include <fstream>
#include <string>
USING_NAMESPACE(std)

#include <AH/Error/Class.h>
#include <AH/Error/Basic.h>
#include <AH/IO/Convert.h>
#include <AH/IO/Riff/WaveFile.h>
#include <AH/IO/Riff/WaveFileCached.h>
USING_NAMESPACE(AH)

// --- local constants ---

static const char* cmdLineOptions = "hnpstvV";
static const char* version = "0.8";

// reduced as cygwin has problems with big arrays on the stack
// static const uint32 MAX_SAMPLES = 256 * 1024;  // 256k
static const uint32 MAX_SAMPLES = 64 * 1024;  // 64k

// --- local variables ---

static int verbose = 0;
static bool normalize = false;
static bool printTime = false;
static bool printTocPos = false;
static bool printSamNum = false;

// --- local classes ---

// --- local functions ---

static void usage(bool help = false)
{
    cerr << "Usage: wav2text [-" << cmdLineOptions << "] <audiofile> <textfile>\n"
	 << "       print the data from a WAV (MP3, MP+, Ogg, Pac) file into\n"
	 << "       a text file.\n"
	 << "  options:\n"
	 << "   -h   - print this help message\n"
	 << "   -n   - normalize into float range [-1..1]\n"
	 << "   -t   - print time in seconds\n"
	 << "   -s   - print sample number\n"
	 << "   -p   - print toc position\n"
	 << "   -v   - verbose mode (increase debug level)\n"
	 << "   -V   - print version\n"
	 << "   <audiofile>  - file containing audio data, supported file types:\n"
	 << "                  .wav           : PCM-WAV\n"
	 << "                  .mp3           : MP3\n"
	 << "                  .mpc .mp+ .mpp : MPEG+\n"
	 << "                  .ogg           : OGG-Vorbis\n"
	 << "                  .pac           : LPAC"
	 << endl;
    if (!help) return;

    cerr << "\n"
	 << "Examples:\n"
	 << "  wav2text a.wav a.txt\n"
	 << "  - print just the data in a.wav as text into a.txt\n"
	 << "  wav2text -s a.wav a.txt\n"
	 << "  - print sample number and data in a.wav as text into a.txt\n"
	 << endl;
} // end of usage()


static void print_version()
{
    cerr << "wav2text Vs " << version << " " 
	 << "(c) 2003 Andreas Hnnebeck\n"
	 << "Licensed under the terms of the GNU General Public License.\n"
	 << "Report bugs to andreas@huennebeck-online.de\n"
	 << "http://www.huennebeck-online.de"
	 << endl;
} // end of print_version()


static std::string samples_to_toc(uint32 samples)
{
    uint32 minutes = samples / (44100 * 4 * 60);
    samples %= (44100 * 4 * 60);
    uint32 seconds = samples / (44100 * 4);
    samples %= (44100 * 4);
    uint32 sectors = samples / 2352;

    char buf[32];
    (void)sprintf(buf, "%02d:%02d:%02d", minutes, seconds, sectors);
    return buf;
}


//----------------------------------------------------------------
// On Error: throws an exception
//----------------------------------------------------------------
static void process
(
    const string& inputFileName,
    const string& outputFileName
)
{
    if (2 < verbose) cerr << "process:" << endl;

    // create a wav file object for read:
    if (3 < verbose) cerr << "open input file \"" << inputFileName
			  << "\":" << endl;
    IORiffWaveFileCached in(inputFileName.c_str(), MAX_SAMPLES, verbose);
    if (verbose  > 3) in.printInfo(cerr);

    // create a text file stream for output, print header:
    ofstream out(outputFileName.c_str());
    out << "# ASCII representation of " << inputFileName << "\n";
    out << "# Samplerate / Hz    : " << in.getSampleRate() << "\n";
    out << "# Bits per sample    : " << in.getSampleSize() << "\n";
    out << "# Number of channels : " << in.getChannelNum() << "\n";
    out << "# Play time / sec    : ";
    if (in.isPipe()) out << "unknown (from pipe)\n";
    else             out << in.getTime(in.getSampleNum()) << "\n";
    out << "#\n";
    if (printSamNum) out << "sample\t";
    if (printTime)   out << "time/sec\t";
    if (printTocPos) out << "TOC-position\t";
    for (uint32 chan = 0; chan < in.getChannelNum(); ++chan)
	out << "ch-" << chan+1 << "\t";
    out << "\n";
    out << "#--------------------------------------------------------------\n";

    // how many samples do we have to read:
    uint32 samToRead = in.getSampleNum();
    for (uint32 curSam = 0; curSam < samToRead; ++curSam)
    {
	if (printSamNum) out << curSam+1 << "\t";
	if (printTime) out << in.getTime(curSam) << "\t";
	if (printTocPos) out << samples_to_toc(curSam) << "\t";

	for (uint32 chan = 0; chan < in.getChannelNum(); ++chan)
	{
	    int32 value;
	    if (0 >= in.readSample(&value, curSam, chan))
		break;
	    if (normalize)
		out << value / (double)in.getMaxAmplitude() << "\t";
	    else 
		out << value << "\t";
	}
	out << "\n";
    }
} // end of process()


// --- global functions ---


int main(int argc, char* argv[])
{
    //------------------------------
    // handle command line options:
    //------------------------------
    int option;
    while ((option = getopt(argc, argv, cmdLineOptions)) != -1)
    {
	switch (option) 
	{
	case 'n': normalize = true;     break;
	case 't': printTime = true;     break;
	case 'p': printTocPos = true;   break;
	case 's': printSamNum = true;   break;
	case 'v': verbose++;            break;
	case 'h': usage(true);          exit(0);
	case 'V': print_version();      exit(0);
	default : usage();              exit(1);
	}
    }

    // the remaining arguments are wav file and text file:
    if (optind >= argc) 
    {
	cerr << "Error: missing wav and text files" << endl;
	usage();
	exit(1);
    }

    // the next argument is the wav file:
    string inputFileName(argv[optind++]);

    // the next argument is the text file:
    if (optind >= argc) 
    {
	cerr << "Error: missing text file" << endl;
	usage();
	exit(1);
    }
    string outputFileName(argv[optind++]);

    // check for too many arguments:
    if (optind < argc) 
    {
	cerr << "Error: too many arguments (last valid is \"" 
	     << inputFileName << "\")" << endl;
	usage();
	exit (1);
    }

    //---------------------
    // process input file:
    //---------------------
    try { process(inputFileName, outputFileName); }
    catch(ErrorClass& error)
    {
	cerr << error.getMsg() << endl;
	exit(1);
    }

    exit (0);
    return  (0); // to keep MSVC quiet
} // end of main()


// --- end of file ---
